﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Net;
using System.Xml.Linq;
using Telerik.Windows.Controls;
using System.Windows.Input;

namespace DataBindingCoverFlow
{
	public class FlickrRssViewModel
	{
		private Uri RssUri = new Uri(@"http://api.flickr.com/services/feeds/photos_public.gne?format=rss_2", UriKind.Absolute);

		static FlickrRssViewModel()
		{
			AddImages = new RoutedCommand("AddImages", typeof(FlickrRssViewModel));
		}

		public FlickrRssViewModel()
		{
			this.Items = new ObservableCollection<ImageItem>();

			this.WebClient = new WebClient();
			this.WebClient.DownloadStringCompleted += this.OnDownloadStringCompleted;

			this.DownloadImages();
		}

		public bool IsCurrentlyDownloading
		{
			get; private set;
		}

		public static ICommand AddImages
		{
			get;
			private set;
		}

		public ObservableCollection<ImageItem> Items
		{
			get;
			private set;
		}

		protected WebClient WebClient
		{
			get;
			private set;
		}

		public CommandBindingCollection GetCommandBindings()
		{
			return new CommandBindingCollection()
			{
				new CommandBinding (AddImages, this.OnAddImagesExecuted, this.OnAddImagesCanExecute)
			};
		}

		protected static void UpdateCommandsState()
		{
			CommandManager.InvalidateRequerySuggested();
		}

		private void DownloadImages()
		{
			this.WebClient.DownloadStringAsync(RssUri);

			this.IsCurrentlyDownloading = true;
			UpdateCommandsState();
		}

		private void OnAddImagesExecuted(object sender, ExecutedRoutedEventArgs e)
		{
			this.DownloadImages();
		}

		private void OnAddImagesCanExecute(object sender, CanExecuteRoutedEventArgs e)
		{
			e.CanExecute = !this.IsCurrentlyDownloading;
		}

		private void OnDownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
		{
			XDocument feed = XDocument.Parse(e.Result);
			foreach (XElement entry in feed.Root.Elements(XName.Get("entry", "http://www.w3.org/2005/Atom")))
			{
				string[] links = (from link in entry.Elements(XName.Get("link", "http://www.w3.org/2005/Atom"))
								  where string.Compare(link.Attribute(XName.Get("rel")).Value, "enclosure") == 0
								  select link.Attribute(XName.Get("href")).Value).ToArray<string>();
				if (links.Length > 0)
				{
					this.Items.Add(new ImageItem(links[0]));
				}
			}
			this.IsCurrentlyDownloading = false;
			UpdateCommandsState();
		}
	}
}